'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _constants = require('../constants');

var _optimisticRedux = require('optimistic-redux');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Middleware that intercepts actions created
 * by the createServerAction tool.
 *
 * Once intercepted it dispatches a BEGIN_REQUEST,
 * emits a server_action and waits on a response
 * containing an action with the same requestId.
 *
 * Once it receives a response, dispatch a COMPLETE_REQUEST
 * or a FAIL_REQUEST depending on the actions status.
 */
exports.default = function (socket) {
    return function (store) {

        /* Here we keep some socket related information
         * that can be used to determine if we are
         * connected or not.
         * */
        var state = {
            connected: false,
            complained: false,
            backlog: []
        };

        /* Here we begin listening for a response
         * action with a matching requestId. If
         * such an response is caught, we dispatch
         * a request update action, and we dispatch
         * the action received by the server.
         * */
        var handleResponse = function handleResponse(clientAction) {
            socket.on("server_response", function (serverAction) {
                var meta = serverAction.meta;

                if (clientAction.meta.requestId == meta.requestId) {
                    if (meta.status != _constants.COMPLETE_REQUEST && meta.status != _constants.FAIL_REQUEST) throw new Error("Server responded with something other than an expected request status (COMPLETE_REQUEST | FAIL_REQUEST)");

                    store.dispatch({
                        type: meta.status,
                        payload: {
                            requestId: meta.requestId,
                            requestName: meta.requestName,
                            error: meta.error
                        }
                    });

                    /* If our local action was optimistic, we need
                     * to COMMIT or REVERT is.
                     * */
                    if (clientAction.meta._isOptimistic) {
                        serverAction.optimist = {
                            type: meta.status == _constants.COMPLETE_REQUEST ? _optimisticRedux.COMMIT : _optimisticRedux.REVERT,
                            id: meta.requestId
                        };
                    }

                    if (serverAction.type) {
                        store.dispatch(serverAction);
                    } else if (clientAction.meta._isOptimistic) {
                        serverAction.type = "@@request-layer/COMMIT_OR_REVERT";
                        store.dispatch(serverAction);
                    }
                }
            });
        };

        /* A wrapper for socket.emit that first checks
         * the current connection status of the socket
         * and backlogs the action if the socket is
         * not connected
         * */
        var emit = function emit(action) {
            if (state.connected) {
                socket.emit("server_action", action);
            } else {
                state.backlog.push(action);

                /* If we stay disconnected for more than 5 seconds
                 * we can assume we do not have an internet connection
                 * and emit a NO_INTERNET action
                 * */
                setTimeout(function () {
                    if (!state.complained && !state.connected) {
                        state.complained = true;
                        store.dispatch({
                            type: _constants.NO_INTERNET
                        });
                    }
                }, 5000);
            }
        };

        /* On reconnect, update our state accordingly
         * and re-emit any backlogged actions that were
         * created during the period of no connection.
         * */
        socket.on("connect", function () {
            var backlog = state.backlog;

            state.backlog = [];
            state.connected = true;

            (0, _forEach3.default)(backlog, emit);
        });
        socket.on("disconnect", function () {
            state.connected = false;
            state.complained = false;
        });

        return function (next) {
            return function (action) {
                var meta = action.meta;

                /* Determine if this action is a serverAction */

                if (meta && meta._isServerAction) {

                    /* Dispatch a BEGIN_REQUEST action containing the
                     * action requestId, requestName and any additional
                     * properties provided by the user.
                     * */
                    store.dispatch({
                        type: _constants.BEGIN_REQUEST,
                        payload: _extends({}, meta._metaPayload, {
                            requestId: meta.requestId,
                            requestName: meta.requestName
                        })
                    });

                    handleResponse(action);
                    emit(action);

                    /* If the action wants to be optimistic then we
                     * add optimistic properties to it.
                     * */
                    if (meta._isOptimistic) {
                        action.optimist = { type: _optimisticRedux.BEGIN, id: meta.requestId };
                    }
                }

                return next(action);
            };
        };
    };
};