# Request Layer

This is the request layer used here at One Day Only to handle request tracking between clients and server, as well as optimistic updates.

## The general idea:

We do not directly make api requests from our clients but rather we pass actions via a socket server, which will later respond with the data we need. This is great, but due to its very
asynchronous nature, its hard to keep track of requests. What this layer does is wrap each action with some meta information including a `requestId` and a `requestName` and then 
stores each request in state along with its current status (`pending`, `complete`, `failed`).

## How to use

We need to target 4 areas of the system:

+ Redux middleware - to intercept wrapped actions. sure we could make thunk actions, but its easier to understand whats going on when the action system is sync.
+ Action creators - to wrap an action with the required meta data.
+ Reducers - to handle state mutations and provide optimistic updates.
+ Socket Server - to interpret the clientAction and properly format a responseAction.

```javascript
/* Middleware */
import { requestMiddleware } from '@odo/request-layer'
import io from 'socket.io-client'

const socket = io("http:// ...")
const store = createStore(
    applyMiddleware(requestMiddleware(socket))
)
```

```javascript
/* Action Creators */
import { serverAction } from '@odo/request-layer'

export const someAction = (payload: Object, getRequestId: Function) => serverAction({
    type: String,
    payload,
    meta: {
        optimistic: Boolean,
        requestName: String
    }
}, getRequestId)
```

The reducer provided is actually a _reducer enhancer_ meaning it will wrap your reducers and quietly change state. The reducerEnhancer will also enhance your store with a 
redux-optimist reducer
```javascript
/* Reducer */
import { reducerEnhancer } from '@odo/request-layer'

const roodReducer = combineReducers({ ... })
export default reducerEnhancer(roodReducer)
```